@@ -3,8 +3,7 @@
 import sys
 import os, shutil
 import curses
-import curses.textpad
-import urllib
+import urllib, urlparse
 import timeoutsocket
 import locale
 import time
@@ -12,16 +11,21 @@ import string, StringIO
 import re
 import sha
 import textwrap
-import ConfigParser
+import cPickle
+import threading
 
+#import ctypescurses._ctypescurses as curses
+
+# local modules
+from version import version as VERSION
+import config
+import textfield
 
 # 3rd party modules
 import feedparser
 import html2text
 
 
-import threading
-
 # use_default_colors was added in python 2.4
 # if we don't have it, don't bother with colors
 if not hasattr(curses, "use_default_colors"):
@@ -40,23 +44,13 @@ else:
     BGCOLOR = -1
     USE_COLORS = 1
 
+
 reload(sys)
+# FIXME: fetch this value from config
 sys.setdefaultencoding('utf-8')
-#lang, encoding = locale.getdefaultlocale()
-
-timeoutsocket.setDefaultSocketTimeout(20)
-
-
-BROWSER = r"lynx %s"
-COLOR = {}
-VERSION = "0.9.5"
-RCDIR = "%s/.goodnews-%s" % (os.getenv("HOME"), VERSION)
-
-OS = sys.platform
-NULL = None
 
 # Set the User-Agent string
-USER_AGENT = "Goodnews/%s (%s)" % (VERSION, OS)
+USER_AGENT = "Goodnews/%s (%s)" % (VERSION, sys.platform)
 
 class GoodnewsURLopener(urllib.FancyURLopener):
     version = USER_AGENT
@@ -66,8 +60,6 @@ urllib._urlopener = GoodnewsURLopener()
 CMD_SLOTS_COLUMNS =6 
 CMD_SLOTS = (CMD_SLOTS_COLUMNS * 2) - 1
 
-SHOW_BOZO = 0
-CURSOR_ON = 1
 
 cur_cmd_grp = []
 keybinding = {}
@@ -121,45 +113,9 @@ class NewsItem:
         self.total_pages = 0
 
 
-
-SYSTEM_DEFAULTS = { "HOME": os.environ.get('HOME'),
-                    "VERSION": "ng-0.9.3",
-
-                    "browser": 'lynx %s',
-                    "cachedir": "%(home)/.goodnews-%(version)/cache"
-                    }
-
-
-class Config:
-
-    def __main__(self, file=''):
-
-        self.file = file
-        self.config = ConfigParser.SafeConfigParser(SYSTEM_DEFAULTS)
-
-        if self.file:
-            self.config.read(self.file)
-
-    def get(self, section, option):
-        if has_option(section, option):
-            return self.config.get(section, option)
-        else:
-            return self.config.defaults().get(option)
-        
-    def set(self, section, option,value):
-        self.config.set(section, option, value)
-
-    def write(self):
-        tempfp, tempfname = tempfile.mkstemp(prefix='goodnews-')
-        self.config.write(tempfp)
-        self.move(tempfname, self.file)
-
-
-
 def ex_entry_field(stdscr, default=""):
 
     LINES, COLS = stdscr.getmaxyx()
-
     
     win = curses.newwin(3, COLS -4)
 
@@ -170,22 +126,14 @@ def ex_entry_field(stdscr, default=""):
     text_win = curses.newwin(1,COLS - 6)
     text_win.mvwin((LINES//2) - 2 + 1, 3)
     
-    
-    textpad  = curses.textpad.Textbox(text_win)
+    textpad  = textfield.Textfield(text_win, default)
+    t = textpad.run()
 
-    if default:
-        text_win.addstr(0, 0, default)
-    
-    #curses.echo()
-    #curses.curs_set(1)
-    
-    t = textpad.edit()
-
-    #curses.curs_set(0)
+    stdscr.clear()
     
     return t
-    
 
+    
 def InitCurses():
     
     stdscr = curses.initscr()
@@ -221,7 +169,6 @@ def InitCurses():
     return stdscr
     
 def draw_status(stdscr, txt, delay=0):
-    #FIXME: center all messages in status
     LINES, COLS = stdscr.getmaxyx()
     columns = COLS -1
     stdscr.attron (curses.A_BOLD)
@@ -230,9 +177,9 @@ def draw_status(stdscr, txt, delay=0):
     stdscr.addstr(LINES-3, 0, ' '*COLS)
 
     if USE_COLORS:
-        stdscr.addnstr (LINES-3, 1, txt, COLS-2, curses.color_pair(10));
+        stdscr.addstr (LINES-3, 1, txt[:COLS-2], curses.color_pair(10));
     else:
-        stdscr.addnstr (LINES-3, 1, txt, COLS-2);
+        stdscr.addstr (LINES-3, 1, txt[COLS-2])
 
     stdscr.attroff (curses.A_BOLD);
     stdscr.refresh()
@@ -240,15 +187,10 @@ def draw_status(stdscr, txt, delay=0):
     if delay:
         time.sleep(delay)
 
-
-
-
-
     
 
 def draw_header (stdscr, filterstring="", use_colors=USE_COLORS):
     LINES, COLS = stdscr.getmaxyx()
-    #stdscr.attron (curses.A_STANDOUT)
 
     headerline = ""
     
@@ -263,7 +205,7 @@ def draw_header (stdscr, filterstring="", use_colors=USE_COLORS):
     if filterstring:
         stdscr.addstr(0, COLS-len(filterstring)-1, filterstring, attrs);
 
-    if SHOW_BOZO:
+    if cfg.get_bozo():
         stdscr.addstr(0, len(USER_AGENT) + 10, "*bozo*", attrs)
         
     stdscr.refresh()
@@ -300,8 +242,6 @@ def draw_footer(stdscr, cur_grp, bindings):
             else:
                 outstr = "no cmd"
             
-            #outstr = str(slotspace) + " Cmd Here"
-
             stdscr.attron (curses.A_STANDOUT)
             stdscr.addstr(y, x * slotspace, repr(i))
             stdscr.attroff (curses.A_STANDOUT)
@@ -314,10 +254,15 @@ def draw_footer(stdscr, cur_grp, bindings):
         
             stdscr.addstr(y, (x * slotspace) + len(repr(i)) + 1, outstr)
 
-
         
 
 def dump_traceback(stdscr, s):
+    configdir = cfg.get_configdir()
+    try:
+        open(configdir + '/last_traceback_dump.txt', 'w+').write(s)
+    except:
+        pass
+    
     stdscr.move(2,0)
     stdscr.addstr(s)
     draw_status(stdscr, "****** CRASH CRASH CRASH ******\n****** Press Any Key To Continue ******", 0)
@@ -335,7 +280,6 @@ def keybind_morecmds(stdscr, **args):
     else:
         return
 
-    
     slots = CMD_SLOTS
     
     kys = bindings.keys()
@@ -346,7 +290,6 @@ def keybind_morecmds(stdscr, **args):
         if not type(j) == type(0):
             k.append(j)
 
-
     i = k.index(cur_cmd_grp[-2])
 
     if not cur_cmd_grp[-1] == '^O' or cur_cmd_grp[-2] == k[-1]:
@@ -358,25 +301,11 @@ def keybind_morecmds(stdscr, **args):
     else:
         cur_cmd_grp = k[:slots + 1]        
 
-        
     draw_footer(stdscr, cur_cmd_grp, bindings)
 
 
-def keybind_cursor(stdscr, **args):
-    global CURSOR_ON
-
-    CURSOR_ON = not CURSOR_ON
-
-    if CURSOR_ON:
-        curses.curs_set(0)
-    else:
-        curses.curs_set(1)
-
-
 def keybind_bozo(stdscr, **args):
-    global SHOW_BOZO
-    
-    SHOW_BOZO = not SHOW_BOZO
+    cfg.set_bozo(not cfg.get_bozo())
     
 def keybind_quit(scr, **args):
     return 1
@@ -400,21 +329,44 @@ def keybind_delfeed(stdscr, **args):
         keybind_prev(stdscr)
     
     first_ptr.remove(deletion)
-    
+    stdscr.clear()
 
 def keybind_allread(stdscr, **args):
     for i in highlighted.items:
         highlighted.items[i].readstatus = 1
     pass
 
-def keybind_browser(stdscr, **args):
-    global BROWSER
 
+def keybind_browser(stdscr, **args):
     draw_status(stdscr, 'Enter a browser command. Example: lynx "%s" (%s is the URL)', 0)
-    s = ex_entry_field(stdscr, BROWSER)
+    s = ex_entry_field(stdscr, cfg.get_browser())
     
     if s:
-        BROWSER = s
+        cfg.set_browser(s)
+
+    draw_status(stdscr, '', 0)        
+
+def keybind_editfeedurl(stdscr, **args):
+    draw_status(stdscr, 'Edit the feed url', 0)    
+    _url = highlighted.feedurl
+    
+    url = ex_entry_field(stdscr, highlighted.feedurl)
+    if url and url != _url:
+        highlighted.feedurl = url
+        update_feed(stdscr, highlighted)
+    draw_status(stdscr, '', 0)
+    
+def keybind_feedname(stdscr, **args):
+    
+    draw_status(stdscr, 'Enter a feed name', 0)
+    s = ex_entry_field(stdscr, highlighted.title)
+    
+    if s:
+        highlighted.title = s
+    else:
+        highlighted.title = highlighted._title
+
+    draw_status(stdscr, '', 0)
         
 def keybind_moveup(stdscr, **args):
 
@@ -445,22 +397,25 @@ def keybind_reloadall(stdscr, **args):
     update_all_feeds(stdscr, first_ptr, draw_feeds, highlighted, args['bindings'])
 
 
+def keybind_saveall(stdscr, **args):
+    draw_status(stdscr, "Saving all states...", 0)
+    save_all(stdscr)
+    
 def keybind_urljump (stdscr, **args):
     LINES, COLS = stdscr.getmaxyx()
+
     if not syscall:
         return
-    
+
     draw_status(stdscr, "Executing: %s ..."% syscall[:COLS-7], 0)
     ret = os.system("%s 2>/dev/null"% syscall)
+    
     if ret:
         draw_status(stdscr, "Error '%i' from browser command..."% ret, 2)
 
 def keybind_urljump2(stdscr, **args):
     pass
 
-def keybind_changefeedname(stdscr, **args):
-    pass
-
 def keybind_sortfeeds (stdscr, **args):
     pass
 
@@ -488,17 +443,7 @@ def keybind_help(stdscr, **args):
 def keybind_about(stdscr, **args):
     pass
 
-def keybind_next(stdscr, **args):
-    global highlighted
 
-    if not first_ptr:
-        return
-    
-    if first_ptr.index(highlighted) < len(first_ptr) -1:
-        highlighted = first_ptr[first_ptr.index(highlighted) + 1]
-
-
-# new test
 def keybind_next(stdscr, **args):
     global highlighted, first_scr_ptr
 
@@ -515,16 +460,6 @@ def keybind_next(stdscr, **args):
         first_scr_ptr += 1
 
 
-def keybind_prev(stdscr, **args):
-    global highlighted
-
-    if not first_ptr:
-        return
-    
-    if first_ptr.index(highlighted) > 0:
-        highlighted = first_ptr[first_ptr.index(highlighted) - 1]
-
-# new test
 def keybind_prev(stdscr, **args):
     global highlighted, first_scr_ptr
     
@@ -616,12 +551,14 @@ def keybind_select(stdscr, **args):
             break
 
     cur_cmd_grp = setup_cmd_grp(item_keybind)
+    stdscr.clear()
     
     while 1:
         draw_news(stdscr, highlighted)
         draw_footer(stdscr, cur_cmd_grp, item_keybind)        
         if do_main_cmdloop(stdscr, 0, item_keybind):
             cur_cmd_grp = setup_cmd_grp(args['bindings'])
+            stdscr.clear()
             break
         
 
@@ -636,7 +573,6 @@ def setup_cmd_grp(bindings):
         if not type(j) == type(0):
             cmd_grp.append(j)
             
-
     if len(cmd_grp) > CMD_SLOTS:
         grp = cmd_grp[:CMD_SLOTS]
         grp.append('^O')
@@ -652,7 +588,6 @@ def keybind_feed_enter_quit(stdscr, **args):
         if not cur_feed.highlighted.readstatus:
             return 1
     curses.ungetch('q')
-
     return 1
 
 def keybind_feed_select(stdscr, **args):
@@ -668,19 +603,17 @@ def keybind_feed_select(stdscr, **args):
     item_keybind['n'] = 'keybind_feed_down'
 
     item_keybind['\n'] = 'keybind_feed_enter_quit'
-
     item_keybind[' '] = 'keybind_feed_enter_quit'
-    #item_keybind[' '] = 'keybind_feed_next_page'
-
     item_keybind['B'] = 'keybind_browser'
     
     cur_cmd_grp = setup_cmd_grp(item_keybind)
-    
+    stdscr.clear()    
     while 1:
         draw_item(stdscr, cur_feed)
         draw_footer(stdscr, cur_cmd_grp, item_keybind)
         if do_main_cmdloop(stdscr, 0, item_keybind):
             cur_cmd_grp = setup_cmd_grp(args['bindings'])
+            stdscr.clear()
             break
 
 
@@ -718,32 +651,13 @@ def enter_main(stdscr, ptr, keybindings, encoding='latin-1'):
     cur_cmd_grp = setup_cmd_grp(keybindings)
     
     while 1:
-        if forceredraw:
-            pass
-        else:
-            stdscr.move(0,0)
-            stdscr.clrtobot()
-
-            
+        stdscr.move(0,0)
         draw_header(stdscr, catfilter)
-
-        if (filteractivated):
-            pass
-
-        if typeahead:
-            pass
-        for cur_ptr in first_ptr:
-            pass
-
-        if typeahead:
-            pass
-        else:
-            pass
-
         draw_feeds(stdscr, first_ptr, highlighted, keybindings)
         draw_footer(stdscr, cur_cmd_grp, keybindings)
 
         if do_main_cmdloop(stdscr, typeahead, keybindings):
+            stdscr.clear()
             break
 
 
@@ -833,8 +747,6 @@ def test_pretty_print_item(description):
         sys.stdout.write("   " + line + '\n')
    
         
-    
-    
 def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
     ypos = 4
     LINES, COLS = stdscr.getmaxyx()
@@ -844,7 +756,7 @@ def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
     item = feed.highlighted
 
     stdscr.move(0,0)
-    stdscr.clrtobot()
+    stdscr.clear()
 
     draw_header(stdscr, "")
 
@@ -856,7 +768,6 @@ def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
 
         stdscr.addstr(2, 1, des, curses.A_BOLD)
 
-
     stdscr.move(5,4)
     stdscr.move(5,0)
     pretty_print_item(stdscr, item.title.encode(encoding, 'replace'))
@@ -868,7 +779,6 @@ def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
     
     # ok it's read now
     item.readstatus = 1
-    
 
 
 def draw_link(stdscr, item):
@@ -877,19 +787,22 @@ def draw_link(stdscr, item):
     columns = COLS - 10
     maxlines = LINES - 6
 
+    # Clear line
+    stdscr.move(maxlines + 2, 1)
+    stdscr.clrtoeol()
+    
     stdscr.attron(curses.A_BOLD)
     
     syscall = doupdatelink(stdscr, item.link)
 
     #Show only link
     if len(item.link) > columns:
-        stdscr.addstr(maxlines + 2, 1, "-> " + item.link[:columns-5] + "...'")
+        stdscr.addstr(maxlines + 2, 1, "-> " + item.link[:columns-5] + " ...")
     else:
         stdscr.addstr(maxlines + 2, 1, "-> " + item.link)
     stdscr.attroff(curses.A_BOLD)
 
 
-
 def draw_news(stdscr, feed, use_colors=USE_COLORS, encoding='latin-1'):
     global cur_feed
     
@@ -899,7 +812,6 @@ def draw_news(stdscr, feed, use_colors=USE_COLORS, encoding='latin-1'):
     maxlines = LINES - 6
     
     stdscr.move(0,0)
-    stdscr.clrtobot()
 
     draw_header(stdscr, "")
 
@@ -995,7 +907,7 @@ def draw_feeds(stdscr, feeds, highlighted='', keybindings={}):
         else:
             title = feed.link
 
-        if feed.bozo and SHOW_BOZO:
+        if feed.bozo and cfg.get_bozo():
             title = "*bozo* " + title
             
         if len(title) > columns:
@@ -1021,22 +933,21 @@ def draw_feeds(stdscr, feeds, highlighted='', keybindings={}):
         
 
 def doupdatelink(stdscr, link):
+    # FIXME: some weird stuff here, clean up and rewrite
     global syscall
     try:
-        syscall = BROWSER % link
+        syscall = cfg.get_browser() % link
     except TypeError:
         draw_status(stdscr, "Error in browser command!", 1)
-        draw_status(stdscr, "'%s' missing %%s?" % BROWSER, 0)
+        draw_status(stdscr, "'%s' missing %%s?" % cfg.get_browser(), 0)
 
     return syscall
 
             
-
 def do_main_cmdloop(stdscr, typeahead, keybindings):
     global cur_cmd_grp
     quit = 0
 
-
     uiinput = stdscr.getch();
 
     if (typeahead):
@@ -1048,7 +959,9 @@ def do_main_cmdloop(stdscr, typeahead, keybindings):
             try:
                 uiinput = chr(uiinput)
             except ValueError:
-                draw_status(stdscr, repr(repr(uiinput)) + "   ", 0)
+                #draw_status(stdscr, repr(repr(uiinput)) + "   ", 0)
+                pass
+
                 
         if keybindings.has_key(uiinput):
             method = keybindings[uiinput]
@@ -1059,7 +972,8 @@ def do_main_cmdloop(stdscr, typeahead, keybindings):
                 if quit:
                     return 1
         else:
-            draw_status(stdscr, repr(repr(uiinput)) + "   ", 0)
+            #draw_status(stdscr, repr(repr(uiinput)) + "   ", 0)
+            pass
 
 
     if (uiinput == ord('\n')):
@@ -1087,7 +1001,6 @@ def do_main_cmdloop(stdscr, typeahead, keybindings):
 
 class FeedParser:
     def __init__(self, cur_ptr):
-        global SHOW_BOZO
         
         new_items = {}
         downloaded_items ={}
@@ -1101,11 +1014,14 @@ class FeedParser:
         rssDocument = feedparser.parse(cur_ptr.feed_data)
 
         if rssDocument.has_key('bozo') and rssDocument['bozo']:
-            SHOW_BOZO = 1
             cur_ptr.bozo = 1
             
         try:
-            cur_ptr.title = strip_tags(rssDocument['feed']['title'])
+            # FIXME: when user has set title, don't check for updates,
+            # otherwise check every time
+            if not cur_ptr.title or cur_ptr.title == "(no title)":
+                cur_ptr.title = strip_tags(rssDocument['feed']['title'])
+                cur_ptr._title = cur_ptr.title 
         except:
             cur_ptr.title = "(no title)"
 
@@ -1129,6 +1045,10 @@ class FeedParser:
                 
             if item.has_key('link'):
                 link = item['link']
+                if link[0] == '/':
+                    _feed_url = urlparse.urlparse(cur_ptr.link)
+                    link = urllib.quote(link)
+                    link = "://".join(_feed_url[:2]) + link
             else:
                 link = ""
 
@@ -1169,7 +1089,6 @@ class FeedParser:
                 if new_item.id not in orginal_sort:
                     orginal_sort.append(new_item.id)
 
-
                     
         # remove items no longer in print...
         # FIXME: (add an expirery date function here)
@@ -1256,7 +1175,7 @@ def strip_links(txt):
 def clean_up_and_exit(func, error):
     global stdscr
     if not error:
-        write_it_out(stdscr)
+        save_all(stdscr)
 
     stdscr.clear()
     stdscr.refresh()
@@ -1281,107 +1200,50 @@ def clean_up_and_exit(func, error):
 def hashurl(url):
     return url.replace('/', '_')
 
-
+    
 #/* Load config and populate caches. */
 def load_state_config(stdscr):
-    #FIXME: load_state looks a road kill. 
-    global BROWSER, COLOR, keybinding
-    
-    if not os.path.exists(RCDIR):
+
+    global COLOR, keybinding, cfg
+
+    cfg = config.Config()
+    configdir = cfg.get_configdir()
+    cachedir = cfg.get_cachedir()
+    timeoutsocket.setDefaultSocketTimeout(cfg.get_timeout())
+
+
+    if not os.path.exists(configdir):
         try:
-            os.mkdir (RCDIR, 0755)
+            os.mkdir (configdir, 0755)
         except OSError, err:
             clean_up_and_exit ("Creating config directory", err);
 
     else:
-        if not os.path.isdir(RCDIR):
+        if not os.path.isdir(configdir):
             clean_up_and_exit ("Creating config directory",
                           "A file with the same name exists!")
-	
-
-    fname = RCDIR + "/cache" 
         
-    if not os.path.exists(fname):
-        #/* Create directory. */
+    if not os.path.exists(cachedir):
         try:
-            os.mkdir(fname, 0755)
+            os.mkdir(cachedir, 0755)
         except OSError, err:
             clean_up_and_exit ("Creating config directory cache/", err)
     else:
-        if not os.path.isdir(fname):
+        if not os.path.isdir(cachedir):
             clean_up_and_exit ("Creating config directory cache",
                           "A file with the same name exists!")
 
-
-    fname = RCDIR + "/cache-readstatus" 
-        
-    if not os.path.exists(fname):
-        #/* Create directory. */
-        try:
-            os.mkdir(fname, 0755)
-        except OSError, err:
-            clean_up_and_exit ("Creating config directory cache/", err)
-    else:
-        if not os.path.isdir(fname):
-            clean_up_and_exit ("Creating config directory cache",
-                          "A file with the same name exists!")
-
-        
-    
     draw_status (stdscr, "Reading configuration settings...", 0)
-	
-
-
-    # default browser
-    fname = RCDIR + "/browser" 
-
-    try:
-        BROWSER = open(fname).read().strip()
-
-    except IOError:
-        draw_status (stdscr, "Creating new config \"browser\"...", 0)
-
-        try:
-            BROWSER = r'lynx %s'
-            configfile = open(fname, "w+")
-            configfile.write(BROWSER)
-            configfile.close()
-        except IOError, err:
-            clean_up_and_exit ('Create initial configfile "config"', err) #/* Still didn't work? */
-
-
-
 
     # list of feeds
-    fname = RCDIR + "/urls"
-
-    try:
-        #configfile = open(fname)
-        for line in open(fname).readlines():
-            url, custom, cats = line.rstrip().split('|')
-
-            new_ptr = Feed()
-            new_ptr.feedurl = url
-            new_ptr.override = custom
+    #fname = configdir + "/urls"
 
+    for url in cfg.get_feeds():
+        new_ptr = load_feed(stdscr, url)
+        if new_ptr:
             first_ptr.append(new_ptr)
-            #draw_status(stdscr, repr(new_ptr), 1)
-            if cats:
-                for c in cats.split(','):
-                    categories[c].append(new_ptr)
-            
-    except IOError:
-        draw_status (stdscr, "Creating new configfile urls.", 0);
-        try:
-            configfile = open (fname, "w+")
-            configfile.close()            
-        except IOError, err:
-            clean_up_and_exit ('Create initial configfile "urls', err); # /* Still didn't work? */
-
-    
+        
     # defaults keybindings 
-
-    #keybinding['n'] = 'keybind_next'
     keybinding['n'] = 'keybind_next'
     keybinding['p'] = 'keybind_prev'
     keybinding['q'] = 'keybind_quit'
@@ -1389,6 +1251,8 @@ def load_state_config(stdscr):
     keybinding['D'] = 'keybind_delfeed'
     keybinding['m'] = 'keybind_allread'
     keybinding['B'] = 'keybind_browser'
+    keybinding['T'] = 'keybind_feedname'
+    keybinding['E'] = 'keybind_editfeedurl'
     keybinding['P'] = 'keybind_moveup'
     keybinding['N'] = 'keybind_movedown'
     #keybinding['i'] = 'keybind_feedinfo'
@@ -1396,11 +1260,11 @@ def load_state_config(stdscr):
     keybinding['R'] = 'keybind_reloadall'
     keybinding['o'] = 'keybind_urljump'
     #keybinding['O'] = 'keybind_urljump2'
-    #keybinding['c'] = 'keybind_changefeedname'
     #keybinding['s'] = 'keybind_sortfeeds'
     #keybinding['b'] = 'keybind_pup'
     #keybinding[' '] = 'keybind_pdown'
-    keybinding[' '] = 'keybind_select'    
+    keybinding[' '] = 'keybind_select'
+    keybinding['S'] = 'keybind_saveall'   
     #keybinding['C'] = 'keybind_cursor'
     #keybinding['f'] = 'keybind_filter'
     #keybinding['g'] = 'keybind_filtercurrent'
@@ -1430,46 +1294,10 @@ def load_state_config(stdscr):
 
     # FIXME: read keybindings from file here
 	
-    #/***********************
-    #* COLOR suppport code *
-    #***********************/
-	
-    COLOR["changed"] = 0
-    COLOR["newitems"] = 5
-    COLOR["newitemsbold"] = 0
-    COLOR["urljump"] = 4
-    COLOR["urljumpbold"] = 0
-    
     use_colors = USE_COLORS
+
+    # FIXME: read color config from file here
     
-    fname = RCDIR + "/colors"
-    try:
-        #configfile = open(fname)
-        for line in open(fname).readline():
-            if line[0] == "#":
-                continue
-
-            line = line.strip().split(':')
-
-            if line[0] == "enabled":
-                use_colors = int(line[1])
-                
-            elif line[0] == "new item":
-                COLOR["newitems"] = int(line[1])
-                if COLOR["newitems"] > 7:
-                    COLOR["newitemsbold"] = 1
-            elif line[0] == "goto url":                
-                COLOR['urljump'] = int(line[1])
-                if COLOR['urljump'] > 7:
-                    COLOR['urljumlbold'] = 1
-    except IOError:
-        #/* Set color configfile update flag. */
-        COLOR["changed"] = 1
-
-
-    #FIXME: read color config from file here
-    
-
 
 
 class UpdateFeedThread(threading.Thread):
@@ -1539,20 +1367,15 @@ class UpdateFeedThread(threading.Thread):
         #Don't mess up highlight
         self.cur_ptr.highlighted = None
 
-        #if cur_ptr.feed:
-        #    del(cur_ptr.feed)
-
         return 1
 
-
-
-    
         
 # Update feed from server.
 def update_feed(stdscr, cur_ptr):
 
     if not cur_ptr:
         return 0
+
     LINES, COLS = stdscr.getmaxyx()
 
     draw_status(stdscr, "Downloading " + cur_ptr.feedurl[:COLS], 0)
@@ -1569,31 +1392,21 @@ def update_feed(stdscr, cur_ptr):
         draw_status(stdscr,str(err), 1)
         return 1
 
-    if not cur_ptr.title:
-        cur_ptr.title = cur_ptr.feedurl
-    if not cur_ptr.link:
-        cur_ptr.link = cur_ptr.feedurl
-	
     if not cur_ptr.feed:
         return 0
 
     draw_status(stdscr, "Parsing " + cur_ptr.feedurl[:COLS], 0)
     xml = FeedParser(cur_ptr)
+
     if cur_ptr.bozo:
         draw_status(stdscr, "Warning: bozo XML! Maybe not a RSS feed?", 2)
 
-
     #Don't mess up highlight
     cur_ptr.highlighted = None
-    
-    #if cur_ptr.feed:
-    #    del(cur_ptr.feed)
-
+    draw_status(stdscr, "", 0)
     return 1
 
 
-
-
 def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybindings={}):
     for cur_ptr in tuple(first_ptr):
         if update_feed(stdscr, cur_ptr) == 2:
@@ -1604,7 +1417,6 @@ def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybind
             stdscr.refresh()
 
 
-
 def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybindings={}):
     tlock = threading.Lock()
 
@@ -1620,57 +1432,31 @@ def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybind
         threads[-1].setDaemon(0)
         threads[-1].start()
 
-
     # don't return until all threads are finished,
     # or else we risk all kind of bad things :-) 
     while threading.activeCount() > 1:
         pass
 
-
+    draw_status(stdscr, "", 0)
 
 #/* Load feed from disk. And call update_feed if neccessary. */
-def load_feed(stdscr,cur_ptr):
+def load_feed(stdscr,url):
 
-    hashme = hashurl(cur_ptr.feedurl)
-    fname = RCDIR + "/cache/%s" % (hashme)
+    hashme = hashurl(url)
+    fname = cfg.get_configdir() + "/cache/%s.pickle" % (hashme)
 
     try:
         cache = open(fname)
-    except IOError:
-        tmp = "Cache for %s is toast. Reloading from server..." % cur_ptr.feedurl
+        feed = cPickle.load(cache)
+        return feed
+    except:
+        tmp = "Cache for %s is toast. Reloading from server..." % url
         draw_status (stdscr, tmp, 0);
-        if not update_feed(stdscr, cur_ptr):
-            first_ptr.remove(cur_ptr)
-
-        return 0
-
-	
-    #/* Read complete cachefile. */
-    cur_ptr.feed = cache
-    cur_ptr.feed_data = cur_ptr.feed.read()
-    cur_ptr.feed = StringIO.StringIO(cur_ptr.feed_data)    
-    
-    #/* After loading FeedParserize the mess. */
-    xml = FeedParser(cur_ptr)
-    cache.close()
-
-    if cur_ptr.bozo:
-        draw_status(stdscr, "Warning: bozo XML! Maybe not a RSS feed?", 2)
-
-    
-    # get readstatus
-    fname = RCDIR + "/cache-readstatus/%s" % (hashme)
-    cache_readstatus = open(fname)
-    for i in cache_readstatus.readlines():
-        id, status = i.split('\t')
-        if cur_ptr.items.has_key(id):
-            cur_ptr.items[id].readstatus = int(status)
+        feed = Feed()
+        feed.feedurl = url        
+        if update_feed(stdscr, feed):
+            return feed
             
-    #draw_status(stdscr, cur_ptr.item_link, 1)
-
-    
-    
-    return 1
 
 
 class LoadFeedThread(threading.Thread):
@@ -1700,7 +1486,7 @@ class LoadFeedThread(threading.Thread):
         self.draw_status_tlock(stdscr, tmp, 0)        
 
         hashme = hashurl(self.cur_ptr.feedurl)
-        fname = RCDIR + "/cache/%s" % (hashme)
+        fname = cfg.get_configdir() + "/cache/%s" % (hashme)
 
         try:
             cache = open(fname)
@@ -1727,7 +1513,7 @@ class LoadFeedThread(threading.Thread):
 
 
         # get readstatus
-        fname = RCDIR + "/cache-readstatus/%s" % (hashme)
+        fname = cfg.get_configdir() + "/cache-readstatus/%s" % (hashme)
         cache_readstatus = open(fname)
         for i in cache_readstatus.readlines():
             id, status = i.split('\t')
@@ -1746,163 +1532,37 @@ def load_all_feeds(stdscr):
         load_feed(stdscr, cur_ptr)
 
 
-
-"""
-# threaded version
-def load_all_feeds(stdscr):
-    tlock = threading.Lock()
-
-    threads = []
-
-    for cur_ptr in tuple(first_ptr):
-        threads.append(LoadFeedThread(tlock, stdscr, cur_ptr))
-        threads[-1].setDaemon(0)
-        threads[-1].start()
-
-    # don't return until all threads are finished,
-    # or else we risk all kind of bad things :-) 
-    while threading.activeCount() > 1:
-        pass
-
-"""
-
-
-
 # write config and cache to disk
-def write_it_out(stdscr):
-    global BROWSER
+def save_all(stdscr):
+    global cfg
     
-    #FIXME: add feature to save each cache any time
     #FIXME: write cache to file more often, not only on exit
     #FIXME: only write when things have changed
     draw_status(stdscr, "Saving settings...", 0)
 
-    fname = RCDIR + "/browser"
-    try:
-        configfile = open (fname, "w")
-    
-        configfile.write(BROWSER)
-        configfile.close()
-
-    except IOError, err:
-        clean_up_and_exit ("Save settings (browser)", err)
-
-    
-    fname = RCDIR + "/urls"
-    if os.path.isfile(fname):
-        shutil.copyfile(fname, fname + ".bak")
-
-    try:
-        configfile = open(fname, "w+")
-    except IOError, errno:
-        clean_up_and_exit ("Save settings (urls)", errno)
+    cfg.add_feeds([ptr.feedurl for ptr in first_ptr])
+    cfg.write()
 
     for cur_ptr in first_ptr:
-        configfile.write(cur_ptr.feedurl)
-        configfile.write("|")
-        if cur_ptr.override:
-            configfile.write(cur_ptr.title)
-        configfile.write("|")
-        if cur_ptr.categories:
-            configfile.write(string.join(cur_ptr.categories, ","))
-        configfile.write('\n')# /* Add newline character. */
-		
-    configfile.close()
+        save_feed_cache(stdscr, cur_ptr)
 
 
-    #* Write cache.
-    for cur_ptr in first_ptr:
-
-        hashme = hashurl(cur_ptr.feedurl);
-        fname = RCDIR + "/cache/%s" % (hashme)
-
-        cache = open(fname, "w+")
-            
-        cache.write(cur_ptr.feed_data)
-        cache.close()
+def save_feed_cache(stdscr, feed):
+    hashme = hashurl(feed.feedurl);
+    fname = cfg.get_configdir() + "/cache/%s.pickle" % (hashme)        
+    cache = open(fname, "wb")
+    cPickle.dump(feed, cache)
+    cache.close()
 
 
-    #* Write cache-readstatus.
-    for cur_ptr in first_ptr:
-
-        hashme = hashurl(cur_ptr.feedurl);
-        fname = RCDIR + "/cache-readstatus/%s" % (hashme)
-
-        cache = open(fname, "w+")
-
-        for item_link in cur_ptr.sorted:
-            cache.write(item_link + '\t')
-            cache.write(str(cur_ptr.items[item_link].readstatus))
-            cache.write('\n')
-
-        cache.close()
-
-
-
-	
-    #/* Write color configfile if needed, */
-    # FIXME: color handling is ugly please remove or fix
-    if COLOR['changed']:
-        fname = RCDIR + "/colors"
-        
-        configfile = open(fname, "w+")
-
-        configfile.write("# Goodnews color definitons\n")
-        configfile.write("# black:0\n")
-        configfile.write("# red:1\n")
-        configfile.write("# green:2\n")
-        configfile.write("# orange:3\n")
-        configfile.write("# blue:4\n")
-        configfile.write("# magenta(tm):5\n")
-        configfile.write("# cyan:6\n")
-        configfile.write("# gray:7\n")
-        configfile.write("# brightred:9\n")
-        configfile.write("# brightgreen:10\n")
-        configfile.write("# yellow:11\n")
-        configfile.write("# brightblue:12\n")
-        configfile.write("# brightmagenta:13\n")
-        configfile.write("# brightcyan:14\n")
-        configfile.write("# white:15\n")
-        configfile.write("enabled:%d\n" % USE_COLORS)
-        configfile.write("new item:%d\n"% COLOR['newitems'])
-        configfile.write("goto url:%d\n"% COLOR['urljump'])
-				
-        configfile.close()
-	
-    return
-
-
-def off_we_go(stdscr, first_ptr, keybinding, encoding):
-
-    while 1:
-        try:
-            enter_main(stdscr, first_ptr, keybinding)
-            break
-        except KeyboardInterrupt:
-            break
-        except SystemExit:
-            break
-        except:
-            import traceback, StringIO, sys
-            s = StringIO.StringIO()
-            sys.last_type = sys.exc_type
-            sys.last_value = sys.exc_value
-            sys.last_traceback = sys.exc_traceback.tb_next
-            traceback.print_exception(sys.last_type, sys.last_value, sys.last_traceback, file=s)
-
-            dump_traceback(stdscr, s.getvalue())
-        
-
-def set_default_browser(browser):
-    global BROWSER
-    BROWSER = browser
-
-def add_feed(stdscr, first_ptr):
+def add_feed(stdscr, first_ptr, default=""):
     LINES, COLS = stdscr.getmaxyx()
 
     draw_status(stdscr, "Enter URL of the feed you want to add. Blank line to abort.", 0);
 
     url = ex_entry_field(stdscr)
+
+    draw_status(stdscr, "")
     
     if not url:
         return 0
@@ -1921,8 +1581,6 @@ def add_feed(stdscr, first_ptr):
     return 1
 
 
-
-
 def run(argv):
     global stdscr
     #FIXME: argv checking
@@ -1933,7 +1591,7 @@ def run(argv):
         # load configs and caches
         load_state_config(stdscr);
 
-        load_all_feeds(stdscr);        
+        #load_all_feeds(stdscr);        
         enter_main(stdscr, first_ptr, keybinding)
 
     except KeyboardInterrupt:
@@ -1951,7 +1609,6 @@ def run(argv):
         dump_traceback(stdscr, s.getvalue())
         
 
-    #off_we_go(stdscr, first_ptr, keybinding, encoding)
     clean_up_and_exit("","")
 
 
